"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SegmentClient = void 0;

var _deepmerge = _interopRequireDefault(require("deepmerge"));

var _promise = _interopRequireDefault(require("promise.allsettled"));

var _reactNative = require("react-native");

var _constants = require("./constants");

var _context = require("./context");

var _events = require("./events");

var _InjectContext = require("./plugins/InjectContext");

var _InjectUserInfo = require("./plugins/InjectUserInfo");

var _SegmentDestination = require("./plugins/SegmentDestination");

var _storage = require("./storage");

var _timeline = require("./timeline");

var _types = require("./types");

var _util = require("./util");

var _uuid = require("./uuid");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

class SegmentClient {
  // the config parameters for the client - a merge of user provided and default options
  // Storage
  // current app state
  // subscription for propagating changes to appState
  // logger
  // internal time to know when to flush, ticks every second
  // unsubscribe watchers for the store
  // whether the user has called cleanup
  // Watchables

  /**
   * Access or subscribe to client context
   */

  /**
   * Access or subscribe to adTrackingEnabled (also accesible from context)
   */

  /**
   * Access or subscribe to integration settings
   */

  /**
   * Access or subscribe to destination filter settings
   */

  /**
   * Access or subscribe to user info (anonymousId, userId, traits)
   */

  /**
   * Returns the plugins currently loaded in the timeline
   * @param ofType Type of plugins, defaults to all
   * @returns List of Plugin objects
   */
  getPlugins(ofType) {
    const plugins = { ...this.timeline.plugins
    };

    if (ofType !== undefined) {
      return [...(plugins[ofType] ?? [])];
    }

    return [...this.getPlugins(_types.PluginType.before), ...this.getPlugins(_types.PluginType.enrichment), ...this.getPlugins(_types.PluginType.utility), ...this.getPlugins(_types.PluginType.destination), ...this.getPlugins(_types.PluginType.after)] ?? [];
  }
  /**
   * Retrieves a copy of the current client configuration
   */


  getConfig() {
    return { ...this.config
    };
  }

  constructor(_ref) {
    let {
      config,
      logger,
      store
    } = _ref;

    _defineProperty(this, "config", void 0);

    _defineProperty(this, "store", void 0);

    _defineProperty(this, "appState", 'unknown');

    _defineProperty(this, "appStateSubscription", void 0);

    _defineProperty(this, "logger", void 0);

    _defineProperty(this, "flushInterval", null);

    _defineProperty(this, "watchers", []);

    _defineProperty(this, "destroyed", false);

    _defineProperty(this, "isAddingPlugins", false);

    _defineProperty(this, "timeline", void 0);

    _defineProperty(this, "pendingEvents", []);

    _defineProperty(this, "pluginsToAdd", []);

    _defineProperty(this, "isInitialized", false);

    _defineProperty(this, "isContextLoaded", false);

    _defineProperty(this, "onContextLoadedObservers", []);

    _defineProperty(this, "onPluginAddedObservers", []);

    _defineProperty(this, "platformPlugins", [new _InjectUserInfo.InjectUserInfo(), new _InjectContext.InjectContext()]);

    _defineProperty(this, "context", void 0);

    _defineProperty(this, "adTrackingEnabled", void 0);

    _defineProperty(this, "settings", void 0);

    _defineProperty(this, "filters", void 0);

    _defineProperty(this, "userInfo", void 0);

    _defineProperty(this, "deepLinkData", void 0);

    this.logger = logger;
    this.config = config;
    this.store = store;
    this.timeline = new _timeline.Timeline(); // Initialize the watchables

    this.context = {
      get: this.store.context.get,
      set: this.store.context.set,
      onChange: this.store.context.onChange
    };
    this.adTrackingEnabled = {
      get: (0, _storage.createGetter)(() => {
        var _this$store$context$g, _this$store$context$g2;

        return ((_this$store$context$g = this.store.context.get()) === null || _this$store$context$g === void 0 ? void 0 : (_this$store$context$g2 = _this$store$context$g.device) === null || _this$store$context$g2 === void 0 ? void 0 : _this$store$context$g2.adTrackingEnabled) ?? false;
      }, async () => {
        var _context$device;

        const context = await this.store.context.get(true);
        return (context === null || context === void 0 ? void 0 : (_context$device = context.device) === null || _context$device === void 0 ? void 0 : _context$device.adTrackingEnabled) ?? false;
      }),
      onChange: callback => this.store.context.onChange(context => {
        var _context$device2;

        callback((context === null || context === void 0 ? void 0 : (_context$device2 = context.device) === null || _context$device2 === void 0 ? void 0 : _context$device2.adTrackingEnabled) ?? false);
      })
    };
    this.settings = {
      get: this.store.settings.get,
      onChange: this.store.settings.onChange
    };
    this.filters = {
      get: this.store.filters.get,
      onChange: this.store.filters.onChange
    };
    this.userInfo = {
      get: this.store.userInfo.get,
      set: this.store.userInfo.set,
      onChange: this.store.userInfo.onChange
    };
    this.deepLinkData = {
      get: this.store.deepLinkData.get,
      onChange: this.store.deepLinkData.onChange
    }; // Watch for isReady so that we can handle any pending events
    // Delays events processing in the timeline until the store is ready to prevent missing data injected from the plugins

    this.store.isReady.onChange(value => this.onStorageReady(value)); // add segment destination plugin unless
    // asked not to via configuration.

    if (this.config.autoAddSegmentDestination) {
      const segmentDestination = new _SegmentDestination.SegmentDestination();
      this.add({
        plugin: segmentDestination
      });
    } // Setup platform specific plugins


    this.platformPlugins.forEach(plugin => this.add({
      plugin: plugin
    }));
  }
  /**
   * Initializes the client plugins, settings and subscribers.
   * Can only be called once.
   */


  async init() {
    if (this.isInitialized) {
      this.logger.warn('SegmentClient already initialized');
      return;
    }

    await this.fetchSettings(); // flush any stored events

    this.flush(false); // set up the timer/subscription for knowing when to flush events

    this.setupInterval();
    this.setupStorageSubscribers(); // set up tracking for lifecycle events

    this.setupLifecycleEvents(); // check if the app was opened from a deep link

    await this.trackDeepLinks(); // save the current installed version

    await this.checkInstalledVersion();
    this.isInitialized = true;
  }

  generateFiltersMap(rules) {
    const map = {};

    for (const r of rules) {
      const key = r.destinationName ?? _constants.workspaceDestinationFilterKey;
      map[key] = r;
    }

    return map;
  }

  async fetchSettings() {
    const settingsEndpoint = `${_constants.settingsCDN}/${this.config.writeKey}/settings`;

    try {
      var _resJson$middlewareSe;

      const res = await fetch(settingsEndpoint);
      const resJson = await res.json();
      const integrations = resJson.integrations;
      const filters = this.generateFiltersMap(((_resJson$middlewareSe = resJson.middlewareSettings) === null || _resJson$middlewareSe === void 0 ? void 0 : _resJson$middlewareSe.routingRules) ?? []);
      this.logger.info(`Received settings from Segment succesfully.`);
      await Promise.all([this.store.settings.set(integrations), this.store.filters.set(filters)]);
    } catch {
      this.logger.warn(`Could not receive settings from Segment. ${this.config.defaultSettings ? 'Will use the default settings.' : 'Device mode destinations will be ignored unless you specify default settings in the client config.'}`);

      if (this.config.defaultSettings) {
        await this.store.settings.set(this.config.defaultSettings.integrations);
      }
    }
  }
  /**
   * Clears all subscriptions to the store
   */


  unsubscribeStorageWatchers() {
    if (this.watchers.length > 0) {
      for (const unsubscribe of this.watchers) {
        try {
          unsubscribe();
        } catch (e) {
          this.logger.error(e);
        }
      }
    }
  }
  /**
   * There is no garbage collection in JS, which means that any listeners, timeouts and subscriptions
   * would run until the application closes
   *
   * This method exists in case the user for some reason needs to recreate the class instance during runtime.
   * In this case, they should run client.cleanup() to destroy the listeners in the old client before creating a new one.
   *
   * There is a Stage 3 EMCAScript proposal to add a user-defined finalizer, which we could potentially switch to if
   * it gets approved: https://github.com/tc39/proposal-weakrefs#finalizers
   */


  cleanup() {
    var _this$appStateSubscri;

    if (this.flushInterval) {
      clearInterval(this.flushInterval);
    }

    this.unsubscribeStorageWatchers();
    (_this$appStateSubscri = this.appStateSubscription) === null || _this$appStateSubscri === void 0 ? void 0 : _this$appStateSubscri.remove();
    this.destroyed = true;
    this.isInitialized = false;
  }

  setupInterval() {
    if (this.flushInterval !== null && this.flushInterval !== undefined) {
      clearInterval(this.flushInterval);
    }

    this.flushInterval = setTimeout(() => {
      this.flush();
    }, this.config.flushInterval * 1000);
  }

  setupStorageSubscribers() {
    this.unsubscribeStorageWatchers();
  }

  setupLifecycleEvents() {
    var _this$appStateSubscri2;

    (_this$appStateSubscri2 = this.appStateSubscription) === null || _this$appStateSubscri2 === void 0 ? void 0 : _this$appStateSubscri2.remove();
    this.appStateSubscription = _reactNative.AppState.addEventListener('change', nextAppState => {
      this.handleAppStateChange(nextAppState);
    });
  }
  /**
     Applies the supplied closure to the currently loaded set of plugins.
     NOTE: This does not apply to plugins contained within DestinationPlugins.
      - Parameter closure: A closure that takes an plugin to be operated on as a parameter.
   */


  apply(closure) {
    this.timeline.apply(closure);
  }
  /**
   * Adds a new plugin to the currently loaded set.
   * @param {{ plugin: Plugin, settings?: IntegrationSettings }} Plugin to be added. Settings are optional if you want to force a configuration instead of the Segment Cloud received one
   */


  add(_ref2) {
    let {
      plugin,
      settings
    } = _ref2;

    // plugins can either be added immediately or
    // can be cached and added later during the next state update
    // this is to avoid adding plugins before network requests made as part of setup have resolved
    if (settings !== undefined && plugin.type === _types.PluginType.destination) {
      this.store.settings.add(plugin.key, settings);
    }

    if (!this.store.isReady.get()) {
      this.pluginsToAdd.push(plugin);
    } else {
      this.addPlugin(plugin);
    }
  }

  addPlugin(plugin) {
    plugin.configure(this);
    this.timeline.add(plugin);
    this.triggerOnPluginLoaded(plugin);
  }
  /**
     Removes and unloads plugins with a matching name from the system.
      - Parameter pluginName: An plugin name.
  */


  remove(_ref3) {
    let {
      plugin
    } = _ref3;
    this.timeline.remove(plugin);
  }

  async process(incomingEvent) {
    const event = (0, _events.applyRawEventData)(incomingEvent);

    if (this.store.isReady.get() === true) {
      return this.timeline.process(event);
    } else {
      this.pendingEvents.push(event);
      return event;
    }
  }

  async trackDeepLinks() {
    if (this.getConfig().trackDeepLinks === true) {
      const deepLinkProperties = await this.store.deepLinkData.get(true);
      this.trackDeepLinkEvent(deepLinkProperties);
      this.store.deepLinkData.onChange(data => {
        this.trackDeepLinkEvent(data);
      });
    }
  }

  trackDeepLinkEvent(deepLinkProperties) {
    if (deepLinkProperties.url !== '') {
      const event = (0, _events.createTrackEvent)({
        event: 'Deep Link Opened',
        properties: { ...deepLinkProperties
        }
      });
      this.process(event);
      this.logger.info('TRACK (Deep Link Opened) event saved', event);
    }
  }
  /**
   * Executes when the state store is initialized.
   * @param isReady
   */


  onStorageReady(isReady) {
    if (isReady) {
      // Add all plugins awaiting store
      if (this.pluginsToAdd.length > 0 && !this.isAddingPlugins) {
        this.isAddingPlugins = true;

        try {
          // start by adding the plugins
          this.pluginsToAdd.forEach(plugin => {
            this.addPlugin(plugin);
          }); // now that they're all added, clear the cache
          // this prevents this block running for every update

          this.pluginsToAdd = [];
        } finally {
          this.isAddingPlugins = false;
        }
      } // Send all events in the queue


      for (const e of this.pendingEvents) {
        this.timeline.process(e);
      }

      this.pendingEvents = [];
    }
  }

  async flush() {
    let debounceInterval = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

    if (this.destroyed) {
      return;
    }

    if (debounceInterval) {
      // Reset interval
      this.setupInterval();
    }

    const promises = [];
    (0, _util.getPluginsWithFlush)(this.timeline).forEach(plugin => {
      promises.push(plugin.flush());
    });
    await (0, _promise.default)(promises);
    return Promise.resolve();
  }

  async screen(name, options) {
    const event = (0, _events.createScreenEvent)({
      name,
      properties: options
    });
    await this.process(event);
    this.logger.info('SCREEN event saved', event);
  }

  async track(eventName, options) {
    const event = (0, _events.createTrackEvent)({
      event: eventName,
      properties: options
    });
    await this.process(event);
    this.logger.info('TRACK event saved', event);
  }

  async identify(userId, userTraits) {
    const event = (0, _events.createIdentifyEvent)({
      userId: userId,
      userTraits: userTraits
    });
    await this.process(event);
    this.logger.info('IDENTIFY event saved', event);
  }

  async group(groupId, groupTraits) {
    const event = (0, _events.createGroupEvent)({
      groupId,
      groupTraits
    });
    await this.process(event);
    this.logger.info('GROUP event saved', event);
  }

  async alias(newUserId) {
    const {
      anonymousId,
      userId: previousUserId
    } = await this.store.userInfo.get(true);
    const event = (0, _events.createAliasEvent)({
      anonymousId,
      userId: previousUserId,
      newUserId
    });
    await this.process(event);
    this.logger.info('ALIAS event saved', event);
  }
  /**
   * Called once when the client is first created
   *
   * Detect and save the the currently installed application version
   * Send application lifecycle events if trackAppLifecycleEvents is enabled
   *
   * Exactly one of these events will be sent, depending on the current and previous version:s
   * Application Installed - no information on the previous version, so it's a fresh install
   * Application Updated - the previous detected version is different from the current version
   * Application Opened - the previously detected version is same as the current version
   */


  async checkInstalledVersion() {
    const context = await (0, _context.getContext)(undefined, this.config);
    const previousContext = this.store.context.get(); // Only overwrite the previous context values to preserve any values that are added by enrichment plugins like IDFA

    await this.store.context.set((0, _deepmerge.default)(previousContext ?? {}, context)); // Only callback during the intial context load

    if (!this.isContextLoaded) {
      this.triggerOnContextLoad(_types.UpdateType.initial);
    }

    this.isContextLoaded = true;

    if (!this.config.trackAppLifecycleEvents) {
      return;
    }

    if ((previousContext === null || previousContext === void 0 ? void 0 : previousContext.app) === undefined) {
      const event = (0, _events.createTrackEvent)({
        event: 'Application Installed',
        properties: {
          version: context.app.version,
          build: context.app.build
        }
      });
      this.process(event);
      this.logger.info('TRACK (Application Installed) event saved', event);
    } else if (context.app.version !== previousContext.app.version) {
      const event = (0, _events.createTrackEvent)({
        event: 'Application Updated',
        properties: {
          version: context.app.version,
          build: context.app.build,
          previous_version: previousContext.app.version,
          previous_build: previousContext.app.build
        }
      });
      this.process(event);
      this.logger.info('TRACK (Application Updated) event saved', event);
    }

    const event = (0, _events.createTrackEvent)({
      event: 'Application Opened',
      properties: {
        from_background: false,
        version: context.app.version,
        build: context.app.build
      }
    });
    this.process(event);
    this.logger.info('TRACK (Application Opened) event saved', event);
  }
  /**
   * AppState event listener. Called whenever the app state changes.
   *
   * Send application lifecycle events if trackAppLifecycleEvents is enabled.
   *
   * Application Opened - only when the app state changes from 'inactive' or 'background' to 'active'
   *   The initial event from 'unknown' to 'active' is handled on launch in checkInstalledVersion
   * Application Backgrounded - when the app state changes from 'inactive' or 'background' to 'active
   *
   * @param nextAppState 'active', 'inactive', 'background' or 'unknown'
   */


  handleAppStateChange(nextAppState) {
    if (this.config.trackAppLifecycleEvents) {
      if (['inactive', 'background'].includes(this.appState) && nextAppState === 'active') {
        var _context$app, _context$app2;

        const context = this.store.context.get();
        const event = (0, _events.createTrackEvent)({
          event: 'Application Opened',
          properties: {
            from_background: true,
            version: context === null || context === void 0 ? void 0 : (_context$app = context.app) === null || _context$app === void 0 ? void 0 : _context$app.version,
            build: context === null || context === void 0 ? void 0 : (_context$app2 = context.app) === null || _context$app2 === void 0 ? void 0 : _context$app2.build
          }
        });
        this.process(event);
        this.logger.info('TRACK (Application Opened) event saved', event);
      } else if (this.appState === 'active' && ['inactive', 'background'].includes(nextAppState)) {
        const event = (0, _events.createTrackEvent)({
          event: 'Application Backgrounded'
        });
        this.process(event);
        this.logger.info('TRACK (Application Backgrounded) event saved', event);
      }
    }

    this.appState = nextAppState;
  }

  reset() {
    let resetAnonymousId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;
    const anonymousId = resetAnonymousId === true ? (0, _uuid.getUUID)() : this.store.userInfo.get().anonymousId;
    this.store.userInfo.set({
      anonymousId,
      userId: undefined,
      traits: undefined
    });
    (0, _util.getPluginsWithReset)(this.timeline).forEach(plugin => plugin.reset());
    this.logger.info('Client has been reset');
  }
  /**
   * Registers a callback for when the client has loaded the device context. This happens at the startup of the app, but
   * it is handy for plugins that require context data during configure as it guarantees the context data is available.
   *
   * If the context is already loaded it will call the callback immediately.
   *
   * @param callback Function to call when context is ready.
   */


  onContextLoaded(callback) {
    this.onContextLoadedObservers.push(callback);

    if (this.isContextLoaded) {
      this.triggerOnContextLoad(_types.UpdateType.initial);
    }
  }

  triggerOnContextLoad(type) {
    this.onContextLoadedObservers.map(f => f === null || f === void 0 ? void 0 : f(type));
  }
  /**
   * Registers a callback for each plugin that gets added to the analytics client.
   * @param callback Function to call
   */


  onPluginLoaded(callback) {
    this.onPluginAddedObservers.push(callback);
  }

  triggerOnPluginLoaded(plugin) {
    this.onPluginAddedObservers.map(f => f === null || f === void 0 ? void 0 : f(plugin));
  }

}

exports.SegmentClient = SegmentClient;
//# sourceMappingURL=analytics.js.map